<?php

require_once("lib-tk/shield.php");
require_once("lib-mbapi/include/modules/HSPCModule.php");
require_once("HSPC/Struct/PP/ProcessBatchContentRequest.php");

class HSPCGateway extends HSPCModule
{
	const PM_FORM_ACTION_VIEW = "view";
	const PM_FORM_ACTION_EDIT = "edit";
	const PM_FORM_ACTION_ADD = "add";

	/**
	 * INPUT:
	 *		gateway
	 *			or
	 *		gateway_id
	 *			or
	 *		billing_account_id
	 *			or
	 *		client_contact_id	- use this parameter when no billing_account_id parameter passed
	 */
	public function __construct(array $params)
	{
		parent::__construct($params);

		if (isset($this->input["gateway"]))
			$this->gateway = $this->input["gateway"];

		if (isset($this->input["billing_account_id"]))
		{
			$this->billing_account = Factory()->ClientBillingAccount($this->input["billing_account_id"]);
			$this->input["client_contact_id"] = $this->billing_account->client_contact_id;
			$this->input["gateway_id"] = $this->billing_account->payment_gateway_id;
			$this->loadPaymentMethodData();
		}

		if (is_null($this->gateway) && isset($this->input["gateway_id"]))
			$this->gateway = Factory()->PaymentGateway($this->input["gateway_id"]);

		if (! in_array($this->getPluginType(), array("BT", "OP")))
			throw new Exception("Can not create instance for object [".get_class($this)."]: Wrong type of the passed payment gateway type");

		if (isset($this->input["client_contact_id"]))
			$this->client_contact = Factory()->ClientContact($this->input["client_contact_id"]);
	}

	public function getPluginType()
	{
		require_once("object_managers/PaymentGatewayManager.php");
		return PaymentGatewayManager::getHSPCModuleType($this->gateway->payment_gateway_type);
	}

	public function getPluginTemplateID()
	{
		return $this->gateway->payment_gateway_module;
	}

	public function getPluginConfig()
	{
		return FALSE === ($config = unserialize($this->decryptData($this->gateway->PaymentGatewayParams["config"]->value))) ? NULL : $config;
	}

	public function collectPluginConfig()
	{
		require_once("HSPC/PaymentProcess.php");

		$hspc_result = HSPCPaymentProcess::getCollectedData("{$this->getPluginType()}_{$this->getPluginTemplateID()}", $this->getPluginConfig());

		if ("" != $hspc_result->data->user_error)
			return $hspc_result->data->user_error;

		$this->gateway->PaymentGatewayParams["config"]->value = $this->encryptData(serialize($hspc_result->data->plugin_data));

		return "";
	}

	public function savePluginConfig()
	{
		require_once("object_managers/PaymentGatewayManager.php");
		PaymentGatewayManager::update($this->gateway);
	}

	public function getPluginConfigForm()
	{
		require_once("HSPC/PaymentProcess.php");
		$hspc_result = HSPCPaymentProcess::getEditForm("{$this->getPluginType()}_{$this->getPluginTemplateID()}", $this->getPluginConfig());
		return $hspc_result->data;
	}

	public function loadPaymentMethodData()
	{
		$this->payment_method_data = $this->getPaymentMethodData($this->billing_account->id);
	}

	public function getPaymentMethodData($billing_account_id)
	{
		$billing_account_params = Factory()->ClientBillingAccount($billing_account_id)->ClientBillingAccountParams;
		return array(
			"public_data" => (FALSE === ($data = unserialize($billing_account_params["hspc_public_data"]->value)) ? NULL : $data),
			"secure_data" => (FALSE === ($data = unserialize($billing_account_params["hspc_secure_data"]->value)) ? NULL : $data),
		);
	}

	public function getPaymentMethodForm($action, $secure_data = NULL, $public_data = NULL)
	{
		require_once("HSPC/PaymentMethod.php");

		$payment_method = $this->getPaymentMethod();
		$client_contact_id = is_null($this->client_contact)
			? NULL
			: $this->client_contact->client_contact_id;

		switch ($action)
		{
			case self::PM_FORM_ACTION_VIEW:
				$form_data = HSPCPaymentMethod::getViewForm(
					$payment_method,
					is_null($secure_data) ? $this->payment_method_data["secure_data"] : $secure_data,
					is_null($public_data) ? $this->payment_method_data["public_data"] : $public_data
				);
				break;
			case self::PM_FORM_ACTION_EDIT:
				$form_data = HSPCPaymentMethod::getEditForm(
					$payment_method,
					is_null($secure_data) ? $this->payment_method_data["secure_data"] : $secure_data,
					is_null($public_data) ? $this->payment_method_data["public_data"] : $public_data
				);
				break;
			case self::PM_FORM_ACTION_ADD:
				$form_data = HSPCPaymentMethod::getAddForm(
					$payment_method,
					$client_contact_id,
					$secure_data,
					$public_data
				);
				break;
			default:
				throw new Exception("Unknown action has been passed");
		}
		return array(
			"payment_method" => $payment_method,
			"form_content" => HSPCTools::stylizePaymentMethodForm($form_data->data->form_content),
		);
	}

	public function collectPaymentMethodForm($public_only = false)
	{
		require_once("HSPC/PaymentMethod.php");

		$payment_method = $this->getPaymentMethod();
		$client_contact_id = is_null($this->client_contact)
			? NULL
			: $this->client_contact->client_contact_id;

		$collected_data = HSPCPaymentMethod::getCollectedData(
			$payment_method,
			$client_contact_id,
			$this->payment_method_data["secure_data"],
			$this->payment_method_data["public_data"],
			$public_only
		);
		return array(
			"payment_method" => $payment_method,
			"collected_data" => $collected_data->data,
		);
	}

	public function getSecureData($data)
	{
		require_once("HSPC/PaymentMethod.php");
		$result = HSPCPaymentMethod::getSecureData($this->getPaymentMethod(), $data);
		return $result->data;
	}

	public function getPublicData($data)
	{
		require_once("HSPC/PaymentMethod.php");
		$result = HSPCPaymentMethod::getPublicData($this->getPaymentMethod(), $data);
		return $result->data;
	}

	public function getPublicNumber()
	{
		require_once("HSPC/PaymentMethod.php");
		$result = HSPCPaymentMethod::getPublicNumber(
			$this->getPaymentMethod(),
			$this->payment_method_data["secure_data"],
			$this->payment_method_data["public_data"]
		);
		return $result->data;
	}

	public function encryptData($value)
	{
		return $this->getCryptor()->encrypt($value);
	}

	public function decryptData($value)
	{
		return $this->getCryptor()->decrypt($value);
	}

	protected function getPaymentMethod()
	{
		require_once("HSPC/PaymentProcess.php");
		return ! is_null($this->payment_method)
			? $this->payment_method
			: $this->payment_method = HSPCPaymentProcess::getSupportedPaymentMethod("{$this->getPluginType()}_{$this->getPluginTemplateID()}");
	}

	private function getCryptor()
	{
		if (! is_null($this->cryptor))
		{
			$this->cryptor->errors = array();
			return $this->cryptor;
		}

		require_once("lib-tk/include/crypt/crypt.php");
		return $this->cryptor = new Crypt($this->cryptor_secret_key);
	}

	protected $plugin_type;
	protected $billing_account;
	protected $client_contact;
	protected $payment_method_data = array(
		"public_data" => NULL,
		"secure_data" => NULL,
	);
	protected $cryptor_secret_key = "3356897nf238hjgfjgwy79692hff62kf9qm4cxnm8vt62";

	private $payment_method;
	private $cryptor;
}
